using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Threading;
using System.Windows.Forms;
using PAT.Common;
using PAT.Common.Classes.Assertion;
using PAT.Common.Classes.BA;
using PAT.Common.Classes.Expressions;
using PAT.Common.Classes.Expressions.ExpressionClass;
using PAT.Common.GUI;
using PAT.CSP.ModelChecking;

namespace PAT.Lib
{

        /// <summary>
    /// A (sorted) Set, which gurranttees the GetID is same for sets with same elements
    /// </summary>
    public class PATGUI : ExpressionValue
    {
        private ModelCheckingForm verifier;
        private SpecificationBase Spec;

        public PATGUI()
        {
         
        }

        public PATGUI(ModelCheckingForm form, SpecificationBase specBase)
        {
            Spec = specBase;
            
            if(form != null)
            {
                verifier = new ModelCheckingForm("PAT GUI Testing", Spec, true);
                SelectAssertion(form.SelectIndex);    
                SelectFairness(form.ComboBox_Fairness.SelectedIndex);
                SelectParallel(form.CheckBox_Parallel.Checked);
                SelectShortestTrace(form.CheckBox_ShortestTrace.Checked);
                SelectVerbose(form.CheckBox_Verbose.Checked);
                //verifier.Show();
                //verifier.Hide();
                //verifier.CreateControl();
                verifier.Show();
                verifier.Hide();


            }
        }

        public void LoadModel(int modelIndex)
        {
            string example;
            switch (modelIndex)
            {
                case 0:
                    example = Common.Examples.Examples.LoadDeadlockFreeDP(3);
                    Spec = new PAT.CSP.ModelChecking.Specification(example);
                    Spec.SpecificationName = "DeadlockFreeDP";
                    break;
                case 1:
                    example = Common.Examples.Examples.LoadRW(3, false);
                    Spec = new PAT.CSP.ModelChecking.Specification(example);
                    Spec.SpecificationName = "ReadWriter";
                    break;
                //more to be added
            }
        }

        public void StartVerifier()
        {
            if(Spec != null)
            {
                verifier = new ModelCheckingForm("PAT GUI Testing", Spec, true);
                verifier.Show();
                verifier.Hide();
            }
        }

        public void StartSimulator()
        {
            if (Spec != null)
            {
                //verifier = new ModelCheckingForm("PAT GUI Testing", SpecProcess, true);
            } 
        }

        public int GetNoOfAssertions()
        {
            if(Spec != null)
            {
                return Spec.AssertionDatabase.Count;    
            }
            else
            {
                return 0;
            }
            
        }

        public void SelectAssertion(int assertion)
        {
            if(verifier != null)
            {
                if(verifier.ListView_Assertions.Enabled)
                {
                    verifier.SelectIndex = assertion;
                    Debug.WriteLine("indexe: " + verifier.SelectIndex);
                    if (assertion != -1)
                    {
                        verifier.ListView_Assertions.Items[assertion].Selected = true;
                        verifier.ResetUIOptions(verifier.ListView_Assertions.Items[assertion], true);
                    }
                    
                    //Debug.WriteLine("sdfsd: " + verifier.ListView_Assertions.SelectedIndices.Count);
                }
            }
        }

        public void Verify()
        {
            if (verifier != null)
            {
                if(verifier.Button_Verify.Enabled)
                {
                    verifier.Button_Verify_Click(null, null); // Button_Verify.PerformClick();

                    if (verifier.Assertion != null)
                    {
                        //Assertion.Action += OnAction;
                        verifier.Assertion.ReturnResult += VerificationFinished;
                        //Assertion.Cancelled += Verification_Cancelled;
                        //Assertion.Failed += MC_Failed;

                        //seconds = 1;
                        //ProgressBar.Value = 0;
                        ////timer.Reset();
                        //startMemroySize = GC.GetTotalMemory(true);
                        ////MCTimer.Start();
                        ////timer.Start();
                        verifier.Assertion.Start();

                        Debug.WriteLine("button: " + verifier.Button_Verify.Text);
                        while (verifier.Button_Verify.Text != "Verify")
                        {
                            Thread.Sleep(500);
                            Debug.WriteLine("button: " + verifier.Button_Verify.Text);
                        }
                    }
                }                
            }
        }

        private void VerificationFinished()
        {
            verifier.Button_Verify.Text = "Verify";
        }

        public void ViewBA()
        {
            if (verifier != null)
            {
                if(verifier.Button_BAGraph.Enabled)
                {
                    verifier.Button_BAGraph.PerformClick();
                }
            }
        }

        public void SimulateWitnessTrace()
        {
            if (verifier != null)
            {
                if (verifier.Button_SimulateWitnessTrace.Enabled)
                {
                    verifier.Button_SimulateWitnessTrace.PerformClick();
                }
            }
        }

        public void SelectFairness(int fairnessIndex)
        {
            if (verifier != null)
            {
                if(verifier.ComboBox_Fairness.Enabled)
                {
                    if (fairnessIndex < verifier.ComboBox_Fairness.Items.Count)
                    {
                        verifier.ComboBox_Fairness.SelectedIndex = fairnessIndex;
                    }    
                }                
            }
        }

        public void SelectParallel(bool isParallel)
        {
            if (verifier != null)
            {
                if(verifier.CheckBox_Parallel.Enabled)
                {
                    verifier.CheckBox_Parallel.Checked = isParallel;
                }
            }
        }

        public void SelectShortestTrace(bool isShortest)
        {
            if (verifier != null)
            {
                if(verifier.CheckBox_ShortestTrace.Enabled)
                {
                    verifier.CheckBox_ShortestTrace.Checked = isShortest;
                }
            }
        }

        public void SelectVerbose(bool isVerbose)
        {
            if (verifier != null)
            {
                if(verifier.CheckBox_Verbose.Enabled)
                {
                    verifier.CheckBox_Verbose.Checked = isVerbose;
                }
            }
        }

        public int GetCounterExampleLenth()
        {
            if (verifier != null)
            {
                if(verifier.Assertion != null &&  verifier.Assertion.CounterExampleTrace != null)
                {
                    return verifier.Assertion.CounterExampleTrace.Count;
                }
            }
            return 0;
        }

            public void SelectClose()
        {
            if (verifier != null)
            {
                verifier.Close();
                verifier = null;
            }
        }

        public override string GetID()
        {
            string returnString = "";
            if (Spec != null)
            {
                returnString += Spec.SpecificationName;
            }

            if (verifier != null)
            {
                returnString += ",";
                returnString += verifier.SelectIndex + ",";
                returnString += verifier.ComboBox_Fairness.SelectedIndex + ",";
                returnString += verifier.ComboBox_Fairness.Enabled + ",";
                returnString += verifier.CheckBox_Parallel.Checked + ",";
                returnString += verifier.CheckBox_Parallel.Enabled + ",";
                returnString += verifier.CheckBox_ShortestTrace.Checked + ",";
                returnString += verifier.CheckBox_ShortestTrace.Enabled + ",";
                returnString += verifier.CheckBox_Verbose.Checked + ",";
                returnString += verifier.CheckBox_Verbose.Enabled + ",";
                returnString += verifier.Button_Verify.Text + ",";
                returnString += verifier.Button_Verify.Enabled + ",";

                if (verifier.Assertion != null)
                {
                    returnString += verifier.Assertion.VerificationResult;
                    
                    if(verifier.Assertion.CounterExampleTrace != null)
                    {
                        returnString += Common.Classes.Ultility.Ultility.PPStringList(verifier.Assertion.CounterExampleTrace);    
                    }                    
                }
            }

            return returnString;
        }


        //override
        public override string ToString()
        {

            string returnString = "Name:";
            if(Spec != null)
            {
                returnString += Spec.SpecificationName;
            }
            else
            {
                returnString += "null";
            }

            if (verifier != null)
            {
                returnString += ",";
                returnString += "SelectedAssertionIndex:" + verifier.SelectIndex + ",";
                returnString += "SelectedFairness:" + verifier.ComboBox_Fairness.SelectedIndex + ",";
                returnString += "FairnessEnabled:" + verifier.ComboBox_Fairness.Enabled + ",";
                returnString += "UseParallel:" + verifier.CheckBox_Parallel.Checked + ",";
                returnString += "ParallelEnabled:" + verifier.CheckBox_Parallel.Enabled + ",";
                returnString += "UseShortestTrace:" + verifier.CheckBox_ShortestTrace.Checked + ",";
                returnString += "ShortestTraceEnabled:" + verifier.CheckBox_ShortestTrace.Enabled + ",";
                returnString += "UseVerbose:" + verifier.CheckBox_Verbose.Checked + ",";
                returnString += "VerboseEnabled:" + verifier.CheckBox_Verbose.Enabled + ",";
                returnString += "VerifyButtonText:" + verifier.Button_Verify.Text + ",";
                returnString += "VerifyButtonEnabled:" + verifier.Button_Verify.Enabled + ",";

                if(verifier.Assertion != null)
                {
                    returnString += "VerificationResult" + verifier.Assertion.VerificationResult;
                    if (verifier.Assertion.CounterExampleTrace != null)
                    {
                        returnString += Common.Classes.Ultility.Ultility.PPStringList(verifier.Assertion.CounterExampleTrace);
                    }   
                }
            }
            
            return returnString;
        }


        //override
        public override ExpressionValue GetClone()
        {
            return new PATGUI(verifier, Spec);
        }
    }

    public class ModelCheckingForm : Form
    {
        public int SelectIndex = -1;
        //chengbin4
        private ListViewItem cb_item = null;
        private bool cb_resetUIvalues = false;

        //public static ModelCheckingForm ModelCheckingFormInstance = null;
        private const string mystring = "(*)";
        private const int CORRECT_ICON = 0;
        private const int WRONG_ICON= 1;
        private const int UNKNOWN_ICON = 2;
        private const int CORRECT_ICON_PRPB = 3;

        private string STOP = "Stop";
        private string VERIFY = "Verify";

        protected SpecificationBase Spec;
        public AssertionBase Assertion;
       
        //private Stopwatch timer;

        private long startMemroySize;

        
        public FairnessType Fairness
        {
            get
            {
                switch (this.ComboBox_Fairness.Text)
                {
                    case "No Fairness":
                        return FairnessType.NO_FAIRNESS;
                    case "Fairness Labels Only":
                        return FairnessType.FAIRNESS_LABEL_ONLY;
                    case "Event Level Weak Fairness":
                        return FairnessType.EVENT_LEVEL_WEAK_FAIRNESS;
                    case "Event Level Strong Fairness":
                        return FairnessType.EVENT_LEVEL_STRONG_FAIRNESS;
                    case "Process Level Weak Fairness":
                        return FairnessType.PROCESS_LEVEL_WEAK_FAIRNESS;
                    case "Process Level Strong Fairness":
                        return FairnessType.PROCESS_LEVEL_STRONG_FAIRNESS;
                    case "Strong Global Fairness":
                        return FairnessType.GLOBAL_FAIRNESS;

                }
                return FairnessType.NO_FAIRNESS;
            }            
        }

        public bool ShortestPath
        {
            get
            {
                return this.CheckBox_ShortestTrace.Checked;
            }
        }

        public ModelCheckingForm(string Name, SpecificationBase spec, bool UseFairness)
        {

            //ModelCheckingFormInstance = this;
            InitializeComponent();

            Button_Verify.Text = VERIFY;
            this.Spec = spec;
            //timer = new Stopwatch();

            int Index = 1;
            foreach (KeyValuePair<string, AssertionBase> entry in Spec.AssertionDatabase)
            {
                ListViewItem item = new ListViewItem(new string[] { "", Index.ToString(), entry.Key });
                
                //remember the type of the assertion
                item.Tag = entry.Value.AssertionType;

                //if the assertion is LTL, the button of the view BA should be enabled.
                BuchiAutomata BA = entry.Value.BA;   

                if(BA != null)
                {
                    if (BA.HasXOperator)
                    {
                        item.SubItems[0].Tag = true;
                    }
                }                
                
                //set the question mark image
                //item.ImageIndex = 2;

                this.ListView_Assertions.Items.Add(item);
                Index++;
            }


            if (Name != "")
            {
                this.Text = this.Text + " - " + Name; 
            }
            
            this.StatusLabel_Text.Text = "Select an Assertion to Start with";

            ComboBox_Fairness.SelectedIndex = 0;

            //if there is no fair events in the system, then remove the "Fairness Label Only" entry.
            if(!spec.HasFairEvent)
            {
                ComboBox_Fairness.Items.RemoveAt(1);
            }

            if(!UseFairness)
            {
                this.ComboBox_Fairness.Visible = false;
                this.Label_Fairness.Visible = false;
            }
        }

        #region "UI Initialization"
        /// <summary>
        /// Required designer variable.
        /// </summary>
        private System.ComponentModel.IContainer components = null;

        /// <summary>
        /// Clean up any resources being used.
        /// </summary>
        /// <param name="disposing">true if managed resources should be disposed; otherwise, false.</param>
        protected override void Dispose(bool disposing)
        {
            if (disposing && (components != null))
            {
                components.Dispose();
            }
            base.Dispose(disposing);
        }

        #region Windows Form Designer generated code

        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
            this.components = new System.ComponentModel.Container();
            System.Windows.Forms.ColumnHeader columnHeader1;
            System.Windows.Forms.ColumnHeader columnHeader3;
            System.Windows.Forms.ColumnHeader columnHeader2;
            this.Button_Verify = new System.Windows.Forms.Button();
            this.MyContextMenuStrip = new System.Windows.Forms.ContextMenuStrip(this.components);
            this.MenuItem_Copy = new System.Windows.Forms.ToolStripMenuItem();
            this.MenuItem_Paste = new System.Windows.Forms.ToolStripMenuItem();
            this.MenuItem_Cut = new System.Windows.Forms.ToolStripMenuItem();
            this.toolStripSeparator1 = new System.Windows.Forms.ToolStripSeparator();
            this.MenuItem_SelectAll = new System.Windows.Forms.ToolStripMenuItem();
            this.MenuItem_Clear = new System.Windows.Forms.ToolStripMenuItem();
            this.toolStripSeparator2 = new System.Windows.Forms.ToolStripSeparator();
            this.MenuItem_SaveAs = new System.Windows.Forms.ToolStripMenuItem();
            this.statusStrip1 = new System.Windows.Forms.StatusStrip();
            this.StatusLabel_Text = new System.Windows.Forms.ToolStripStatusLabel();
            this.ProgressBar = new System.Windows.Forms.ToolStripProgressBar();
            this.Button_SimulateWitnessTrace = new System.Windows.Forms.Button();
            this.ToolTip = new System.Windows.Forms.ToolTip(this.components);
            this.Button_BAGraph = new System.Windows.Forms.Button();
            this.CheckBox_Parallel = new System.Windows.Forms.CheckBox();
            this.Label_Fairness = new System.Windows.Forms.Label();
            this.CheckBox_Verbose = new System.Windows.Forms.CheckBox();
            this.CheckBox_ShortestTrace = new System.Windows.Forms.CheckBox();
            this.GroupBox_Assertions = new System.Windows.Forms.GroupBox();
            this.ListView_Assertions = new System.Windows.Forms.ListView();
            this.GroupBox_Options = new System.Windows.Forms.GroupBox();
            this.ComboBox_Fairness = new System.Windows.Forms.ComboBox();
            this.GroupBox_Output = new System.Windows.Forms.GroupBox();
            this.TextBox_Output = new System.Windows.Forms.RichTextBox();
            this.GroupBox_SelectedAssertion = new System.Windows.Forms.GroupBox();
            this.Label_SelectedAssertion = new System.Windows.Forms.Label();
            //this.MCTimer = new System.Windows.Forms.Timer(this.components);
            columnHeader1 = new System.Windows.Forms.ColumnHeader();
            columnHeader3 = new System.Windows.Forms.ColumnHeader();
            columnHeader2 = new System.Windows.Forms.ColumnHeader();
            this.MyContextMenuStrip.SuspendLayout();
            this.statusStrip1.SuspendLayout();
            this.GroupBox_Assertions.SuspendLayout();
            this.GroupBox_Options.SuspendLayout();
            this.GroupBox_Output.SuspendLayout();
            this.GroupBox_SelectedAssertion.SuspendLayout();
            this.SuspendLayout();
            this.Button_Verify.Name = "Button_Verify";
            this.Button_Verify.UseVisualStyleBackColor = true;
            this.Button_Verify.Click += new System.EventHandler(this.Button_Verify_Click);
            // 
            // MyContextMenuStrip
            // 
            this.MyContextMenuStrip.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.MenuItem_Copy,
            this.MenuItem_Paste,
            this.MenuItem_Cut,
            this.toolStripSeparator1,
            this.MenuItem_SelectAll,
            this.MenuItem_Clear,
            this.toolStripSeparator2,
            this.MenuItem_SaveAs});
            this.MyContextMenuStrip.Name = "MyContextMenuStrip";
            // 
            // MenuItem_Copy
            // 
            this.MenuItem_Copy.Name = "MenuItem_Copy";
            this.MenuItem_Copy.Click += new System.EventHandler(this.MenuItem_Copy_Click);
            // 
            // MenuItem_Paste
            // 
            this.MenuItem_Paste.Name = "MenuItem_Paste";
            // 
            // MenuItem_Cut
            // 
            this.MenuItem_Cut.Name = "MenuItem_Cut";
            // 
            // toolStripSeparator1
            // 
            this.toolStripSeparator1.Name = "toolStripSeparator1";
            // 
            // MenuItem_SelectAll
            // 
            this.MenuItem_SelectAll.Name = "MenuItem_SelectAll";
            this.MenuItem_SelectAll.Click += new System.EventHandler(this.MenuItem_SelectAll_Click);
            // 
            // MenuItem_Clear
            // 
            this.MenuItem_Clear.Name = "MenuItem_Clear";
            this.MenuItem_Clear.Click += new System.EventHandler(this.MenuItem_Clear_Click);
            // 
            // toolStripSeparator2
            // 
            this.toolStripSeparator2.Name = "toolStripSeparator2";
            // 
            // MenuItem_SaveAs
            // 
            this.MenuItem_SaveAs.Name = "MenuItem_SaveAs";
            this.MenuItem_SaveAs.Click += new System.EventHandler(this.MenuItem_SaveAs_Click);
            // 
            // statusStrip1
            // 
            this.statusStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.StatusLabel_Text,
            this.ProgressBar});
            this.statusStrip1.LayoutStyle = System.Windows.Forms.ToolStripLayoutStyle.HorizontalStackWithOverflow;
            this.statusStrip1.Name = "statusStrip1";
            // 
            // StatusLabel_Text
            // 
            this.StatusLabel_Text.Name = "StatusLabel_Text";
            // 
            // ProgressBar
            // 
            this.ProgressBar.Alignment = System.Windows.Forms.ToolStripItemAlignment.Right;
            this.ProgressBar.Name = "ProgressBar";
            // 
            // Button_SimulateWitnessTrace
            // 
            this.Button_SimulateWitnessTrace.Name = "Button_SimulateWitnessTrace";
            this.Button_SimulateWitnessTrace.UseVisualStyleBackColor = true;
            this.Button_SimulateWitnessTrace.Click += new System.EventHandler(this.Button_SimulateCounterExample_Click);
            // 
            // ToolTip
            // 
            this.ToolTip.AutomaticDelay = 1;
            this.ToolTip.AutoPopDelay = 10000;
            this.ToolTip.InitialDelay = 1;
            this.ToolTip.IsBalloon = true;
            this.ToolTip.ReshowDelay = 0;
            this.ToolTip.ToolTipIcon = System.Windows.Forms.ToolTipIcon.Info;
            this.ToolTip.ToolTipTitle = "Information";
            // 
            // Button_BAGraph
            // 
            this.Button_BAGraph.Name = "Button_BAGraph";
            this.Button_BAGraph.UseVisualStyleBackColor = true;
            this.Button_BAGraph.Click += new System.EventHandler(this.Button_BAGraph_Click);
            // 
            // CheckBox_Parallel
            // 
            this.CheckBox_Parallel.Name = "CheckBox_Parallel";
            this.CheckBox_Parallel.UseVisualStyleBackColor = true;
            this.CheckBox_Parallel.CheckedChanged += new System.EventHandler(this.CheckBox_Parallel_CheckedChanged);
            // 
            // Label_Fairness
            // 
            this.Label_Fairness.Name = "Label_Fairness";
            // 
            // CheckBox_Verbose
            // 
            this.CheckBox_Verbose.Name = "CheckBox_Verbose";
            this.CheckBox_Verbose.UseVisualStyleBackColor = true;
            this.CheckBox_Verbose.CheckedChanged += new System.EventHandler(this.CheckBox_Verbose_CheckedChanged);
            // 
            // CheckBox_ShortestTrace
            // 
            this.CheckBox_ShortestTrace.Name = "CheckBox_ShortestTrace";
            this.CheckBox_ShortestTrace.UseVisualStyleBackColor = true;
            // 
            // GroupBox_Assertions
            // 
            this.GroupBox_Assertions.Controls.Add(this.ListView_Assertions);
            this.GroupBox_Assertions.Name = "GroupBox_Assertions";
            this.GroupBox_Assertions.TabStop = false;
            // 
            // ListView_Assertions
            // 
            this.ListView_Assertions.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
            columnHeader3,
            columnHeader2,
            columnHeader1});
            this.ListView_Assertions.FullRowSelect = true;
            this.ListView_Assertions.GridLines = true;
            this.ListView_Assertions.HeaderStyle = System.Windows.Forms.ColumnHeaderStyle.None;
            this.ListView_Assertions.HideSelection = false;
            this.ListView_Assertions.MultiSelect = false;
            this.ListView_Assertions.Name = "ListView_Assertions";
            this.ListView_Assertions.ShowGroups = false;
            this.ListView_Assertions.UseCompatibleStateImageBehavior = false;
            this.ListView_Assertions.View = System.Windows.Forms.View.Details;
            this.ListView_Assertions.DoubleClick += new System.EventHandler(this.ListView_Assertions_DoubleClick);
            this.ListView_Assertions.ItemSelectionChanged += new System.Windows.Forms.ListViewItemSelectionChangedEventHandler(this.ListView_Assertions_ItemSelectionChanged);
            // 
            // GroupBox_Options
            // 
            this.GroupBox_Options.Controls.Add(this.CheckBox_ShortestTrace);
            this.GroupBox_Options.Controls.Add(this.ComboBox_Fairness);
            this.GroupBox_Options.Controls.Add(this.CheckBox_Verbose);
            this.GroupBox_Options.Controls.Add(this.CheckBox_Parallel);
            this.GroupBox_Options.Controls.Add(this.Label_Fairness);
            this.GroupBox_Options.Name = "GroupBox_Options";
            this.GroupBox_Options.TabStop = false;
            // 
            // ComboBox_Fairness
            // 
            this.ComboBox_Fairness.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
            this.ComboBox_Fairness.FormattingEnabled = true;
            this.ComboBox_Fairness.Items.AddRange(new object[] {
            "ComboBox_Fairness.Items",
            "ComboBox_Fairness.Items1",
            "ComboBox_Fairness.Items2",
            "ComboBox_Fairness.Items3",
            "ComboBox_Fairness.Items4",
            "ComboBox_Fairness.Items5",
            "ComboBox_Fairness.Items6"});
            this.ComboBox_Fairness.Name = "ComboBox_Fairness";
            // 
            // GroupBox_Output
            // 
            this.GroupBox_Output.Controls.Add(this.TextBox_Output);
            this.GroupBox_Output.Name = "GroupBox_Output";
            this.GroupBox_Output.TabStop = false;
            // 
            // TextBox_Output
            // 
            this.TextBox_Output.BackColor = System.Drawing.Color.White;
            this.TextBox_Output.Name = "TextBox_Output";
            this.TextBox_Output.ReadOnly = true;
            this.TextBox_Output.MouseDown += new System.Windows.Forms.MouseEventHandler(this.TextBox_Output_MouseUp);
            // 
            // GroupBox_SelectedAssertion
            // 
            this.GroupBox_SelectedAssertion.Controls.Add(this.Label_SelectedAssertion);
            this.GroupBox_SelectedAssertion.Controls.Add(this.Button_Verify);
            this.GroupBox_SelectedAssertion.Controls.Add(this.Button_BAGraph);
            this.GroupBox_SelectedAssertion.Controls.Add(this.Button_SimulateWitnessTrace);
            this.GroupBox_SelectedAssertion.Name = "GroupBox_SelectedAssertion";
            this.GroupBox_SelectedAssertion.TabStop = false;
            // 
            // Label_SelectedAssertion
            // 
            this.Label_SelectedAssertion.Name = "Label_SelectedAssertion";
            // 
            // MCTimer
            // 
            //this.MCTimer.Interval = 1000;
            //this.MCTimer.Tick += new System.EventHandler(this.MCTimer_Tick);
            // 
            // ModelCheckingForm
            // 
            this.AutoScaleMode = System.Windows.Forms.AutoScaleMode.Font;
            this.Controls.Add(this.GroupBox_SelectedAssertion);
            this.Controls.Add(this.GroupBox_Output);
            this.Controls.Add(this.GroupBox_Assertions);
            this.Controls.Add(this.statusStrip1);
            this.Controls.Add(this.GroupBox_Options);
            this.Name = "ModelCheckingForm";
            this.FormClosing += new System.Windows.Forms.FormClosingEventHandler(this.ModelCheckingForm_FormClosing);
            this.MyContextMenuStrip.ResumeLayout(false);
            this.statusStrip1.ResumeLayout(false);
            this.statusStrip1.PerformLayout();
            this.GroupBox_Assertions.ResumeLayout(false);
            this.GroupBox_Options.ResumeLayout(false);
            this.GroupBox_Options.PerformLayout();
            this.GroupBox_Output.ResumeLayout(false);
            this.GroupBox_SelectedAssertion.ResumeLayout(false);
            this.GroupBox_SelectedAssertion.PerformLayout();
            this.ResumeLayout(false);
            this.PerformLayout();

        }

        #endregion

        public System.Windows.Forms.Button Button_Verify;
        public System.Windows.Forms.ContextMenuStrip MyContextMenuStrip;
        public System.Windows.Forms.ToolStripMenuItem MenuItem_SaveAs;
        public System.Windows.Forms.ToolStripMenuItem MenuItem_Copy;
        public System.Windows.Forms.ToolStripMenuItem MenuItem_Paste;
        public System.Windows.Forms.ToolStripMenuItem MenuItem_Cut;
        public System.Windows.Forms.ToolStripMenuItem MenuItem_SelectAll;
        public System.Windows.Forms.ToolStripSeparator toolStripSeparator1;
        public System.Windows.Forms.ToolStripSeparator toolStripSeparator2;
        public System.Windows.Forms.ToolStripMenuItem MenuItem_Clear;
        public System.Windows.Forms.StatusStrip statusStrip1;
        public System.Windows.Forms.ToolStripStatusLabel StatusLabel_Text;
        public System.Windows.Forms.ToolStripProgressBar ProgressBar;
        public System.Windows.Forms.Button Button_SimulateWitnessTrace;
        public System.Windows.Forms.ToolTip ToolTip;
        public System.Windows.Forms.GroupBox GroupBox_Assertions;
        public System.Windows.Forms.ListView ListView_Assertions;
        public System.Windows.Forms.Button Button_BAGraph;
        public System.Windows.Forms.GroupBox GroupBox_Options;
        public System.Windows.Forms.GroupBox GroupBox_Output;
        public System.Windows.Forms.RichTextBox TextBox_Output;
        public System.Windows.Forms.GroupBox GroupBox_SelectedAssertion;
        public System.Windows.Forms.Label Label_SelectedAssertion;
        public System.Windows.Forms.Label Label_Fairness;
        //public System.Windows.Forms.Timer MCTimer;
        public System.Windows.Forms.CheckBox CheckBox_Parallel;
        public System.Windows.Forms.ComboBox ComboBox_Fairness;
        public System.Windows.Forms.CheckBox CheckBox_Verbose;
        public System.Windows.Forms.CheckBox CheckBox_ShortestTrace;
        #endregion




        #region UI Operations

        
        private void TextBox_Output_MouseUp(object sender, MouseEventArgs e)
        {
            if (e.Button == System.Windows.Forms.MouseButtons.Right)
            {
                this.MyContextMenuStrip.Show(this.TextBox_Output, e.Location);    
            }            
        }

        private void MenuItem_Copy_Click(object sender, EventArgs e)
        {
            this.TextBox_Output.Copy();
        }

        private void MenuItem_SelectAll_Click(object sender, EventArgs e)
        {
            this.TextBox_Output.SelectedText = this.TextBox_Output.Text;
        }

        private void MenuItem_Clear_Click(object sender, EventArgs e)
        {
            this.TextBox_Output.Text = "";
        }

        private void MenuItem_SaveAs_Click(object sender, EventArgs e)
        {
            SaveFileDialog svd = new SaveFileDialog();
            svd.Title = "Save Query File";
            svd.Filter = "Text Files|*.txt|All files|*.*";

            if (svd.ShowDialog() == DialogResult.OK)
            {
                TextWriter tr = new StreamWriter(svd.FileName);
                tr.WriteLine(this.TextBox_Output.Text);
                tr.Flush();
                tr.Close();                
            }      
        }

        public void Button_BAGraph_Click(object sender, EventArgs e)//chengbin4 - private
        {
            //chengbin4
            //Contract.Requires(Button_BAGraph.Enabled);
            //Contract.Requires(Button_Verify.Text == VERIFY);
            //Contract.Requires(SpecProcess.AssertionDatabase[Label_SelectedAssertion.Text].BA != null);
            //Contract.Requires(ListView_Assertions.SelectedItems.Count == 1);

            if (this.ListView_Assertions.SelectedItems.Count == 0)
            {
                //MessageBox.Show("Please select an assertion first!", Common.Ultility.Ultility.APPLICATION_NAME, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            
            try
            {
                //this.Cursor = Cursors.WaitCursor;
                //Graph g = SpecProcess.GenerateBAGraph(this.ListView_Assertions.SelectedItems[0].SubItems[2].Text);
                //this.Cursor = Cursors.Default;

                //if (g != null)
                //{
                //    LTL2AutoamataConverter v = new LTL2AutoamataConverter(g.UserData.ToString(), "");
                    //v.Show();
                //}
            }
            catch (Exception ex)
            {
                this.Cursor = Cursors.Default;
                Common.Ultility.Ultility.LogException(ex, Spec);
                //MessageBox.Show("Exception happened: " + ex.Message + "\r\n" + ex.StackTrace, "PAT", MessageBoxButtons.OK, MessageBoxIcon.Error);

            }
        }

        #endregion


        #region Enabled Disable Button
        
        private void DisableAllControls()
        {
            this.Cursor = Cursors.WaitCursor;
            this.TextBox_Output.Cursor = Cursors.WaitCursor;
            ListView_Assertions.Enabled = false;
            this.CheckBox_Verbose.Enabled = false;
            //this.CheckBox_POR.Enabled = false;
            this.CheckBox_Parallel.Enabled = false;

            //Panel_EMC.Enabled = false;
            ComboBox_Fairness.Enabled = false;
          

            Button_BAGraph.Enabled = false;            
            Button_Verify.Text = STOP;
            ProgressBar.Value = 0;
            ProgressBar.Visible = false;
            StatusLabel_Text.Text = "Verification Starts...";
  
            Button_SimulateWitnessTrace.Enabled = false;
            this.CheckBox_ShortestTrace.Enabled = false;
        }


        public void EnableAllControls()//chengbin4 - private
        {            
            ListView_Assertions.Enabled = true;

            this.CheckBox_Parallel.Enabled = true;
            this.CheckBox_Verbose.Enabled = true;            

            //the following three values 
            //ComboBox_Fairness.Enabled = true;
            //this.CheckBox_POR.Enabled = true;
            //this.NUD_Depth.Enabled = true;

            Button_Verify.Text = VERIFY;
            ProgressBar.Value = 0;
            ProgressBar.Visible = false;
            this.Cursor = Cursors.Default;
            this.TextBox_Output.Cursor = Cursors.Default;            

            if (this.ListView_Assertions.SelectedItems.Count > 0)
            {
                //ListView_Assertions_ItemSelectionChanged(null, new ListViewItemSelectionChangedEventArgs(this.ListView_Assertions.SelectedItems[0], this.ListView_Assertions.SelectedItems[0].Index, true));
                ResetUIOptions(this.ListView_Assertions.SelectedItems[0], false);
            }

            //MCTimer.Stop();

            //SpecProcess.UnLockSharedData();

        }
        #endregion

        private void ListView_Assertions_DoubleClick(object sender, EventArgs e)
        {
            StartVerification();
        }

        //chengbin4 - cannot change to public because of DisableAllControls() in StartVerification(), so shifted to VerificationResult()
        public void Button_Verify_Click(object sender, EventArgs e)
        {
            //chengbin4
            //Contract.Requires(Button_Verify.Enabled, "button is disabled");
            //Contract.Requires(Button_Verify.Text == VERIFY, "verify test is wrong");
            //Contract.Requires(Label_SelectedAssertion != null && !Label_SelectedAssertion.Text.Equals(""));
            //Contract.Requires(ListView_Assertions.SelectedItems.Count == 1);

            System.Diagnostics.Debug.WriteLine("1111");
            try
            {
                if (Button_Verify.Text == STOP)
                {
                    if (Assertion != null)
                    {
                        Button_Verify.Enabled = false;
                        Assertion.Cancel();
                    }
                }
                else
                {
                    StartVerification();
                }
            }
            catch (Exception ex)
            {
                Common.Ultility.Ultility.LogException(ex, Spec);
            }
        }

        /// <summary>
        /// If there is a special checking or user msg, the sub-class of Model Checking Form should override this method.
        /// </summary>
        /// <returns></returns>
        protected virtual bool ModuleSpecificCheckPassed()
        {
            return true;
        }

        //chengbin4 - thread 1?
        private void StartVerification()
        {
            System.Diagnostics.Debug.WriteLine("2222");

            //chengbin4
            //Contract.Requires(ListView_Assertions.SelectedItems.Count == 1);

            if (SelectIndex == -1)
            {
                //MessageBox.Show("Please select an assertion first!", Common.Ultility.Ultility.APPLICATION_NAME, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (!ModuleSpecificCheckPassed())
            {
                return;
            }

            //if (!SpecProcess.GrabSharedDataLock())
            //{
            //    MessageBox.Show("Please wait for the simulation or parsing finished before verification", Common.Ultility.Ultility.APPLICATION_NAME, MessageBoxButtons.OK, MessageBoxIcon.Error);
            //    return;
            //}
            
            //chengbin4 - concern here
            DisableAllControls();

            try
            {
                System.Diagnostics.Debug.WriteLine("3333");

                //SpecProcess.LockSharedData();

                //these data will not change for SpecProcess.SharedData, so once it is created after parsing, there is no need to set them again.

                PAT.CSP.ModelChecking.Specification specification = Spec as PAT.CSP.ModelChecking.Specification;
                DataStore.DataManager = new DataStore(); // specification.SharedData.DataManager;
                //Specification.AlphaDatabase = specification.SharedData.AlphaDatabase;

                Valuation.VariableLowerBound = specification.SharedData.VariableLowerBound;
                Valuation.VariableUpperLowerBound = specification.SharedData.VariableUpperLowerBound;
                Valuation.HiddenVars = specification.SharedData.LocalVars;
                Valuation.HasVariableConstraints = Valuation.VariableLowerBound.Count > 0 || Valuation.VariableUpperLowerBound.Count > 0;

                Specification.HasSyncrhonousChannel = specification.SharedData.HasSyncrhonousChannel;
                Specification.SyncrhonousChannelNames = specification.SharedData.SyncrhonousChannelNames;
                SpecificationBase.HasAtomicEvent = specification.SharedData.HasAtomicEvent;
                SpecificationBase.HasCSharpCode = specification.SharedData.HasCSharpCode;
                AssertionBase.FairnessType = specification.SharedData.FairnessType;
                AssertionBase.CalculateParticipatingProcess = specification.SharedData.CalculateParticipatingProcess;
                AssertionBase.TimedRefinementAssertion = specification.SharedData.TimedRefinementAssertion;
                //always false
                AssertionBase.ApplyNormalizationInTimedRefinementAssertion = false;

                SpecificationBase.TimedRefinementClockCeiling = specification.SharedData.TimedRefinementClockCeiling;
                SpecificationBase.TimedRefinementClockFloor = specification.SharedData.TimedRefinementClockFloor;

                Common.Ultility.Ultility.CSharpMethods = specification.SharedData.CSharpMethods;
                Common.Ultility.Ultility.CSharpDataType = specification.SharedData.CSharpDataType;


                //ListView_Assertions.SelectedItems[0].ImageIndex = UNKNOWN_ICON;

                Assertion = Spec.AssertionDatabase[ListView_Assertions.Items[SelectIndex].SubItems[2].Text];
                Assertion.Initialize(this, Spec, Fairness, false, this.CheckBox_Verbose.Checked, this.CheckBox_Parallel.Checked, this.ShortestPath);


                //Assertion.Action += OnAction;
                //Assertion.ReturnResult += VerificationFinished;
                //Assertion.Cancelled += Verification_Cancelled;
                //Assertion.Failed += MC_Failed;

                //seconds = 1;
                //ProgressBar.Value = 0;
                ////timer.Reset();
                //startMemroySize = GC.GetTotalMemory(true);
                ////MCTimer.Start();
                ////timer.Start();
                //Assertion.Start();
                System.Diagnostics.Debug.WriteLine("444");
            }
            catch (RuntimeException e)
            {
                Common.Ultility.Ultility.LogRuntimeException(e);
                this.Close();
                return;
            }
            catch (Exception ex)
            {
                //TextBox_Output.Text += ex.Message + "\r\n" + ex.StackTrace;
                //EnableAllControls();
                Common.Ultility.Ultility.LogException(ex, Spec);
                this.Close();
                return;
            }
        }

        private int seconds = 1;
        private void MCTimer_Tick(object sender, EventArgs e)
        {
            StatusLabel_Text.Text = "Verification has been running for " + seconds + "s";
            seconds++;
        }
        public delegate void VerificationFaildeEvents(object sender, System.Threading.ThreadExceptionEventArgs e);

        private void MC_Failed(object sender, System.Threading.ThreadExceptionEventArgs e)
        {

            //if (this.InvokeRequired)
            //{
            //    this.Invoke(new VerificationFaildeEvents(MC_Failed), new object[] { sender, e });
            //}
            //else
            {
                try
                {
                    if (e.Exception is RuntimeException)
                    {

                        Common.Ultility.Ultility.LogRuntimeException(e.Exception as RuntimeException);

                        if (Assertion != null)
                        {
                            TextBox_Output.Text = "Total transitions visited beofre the exception:" + Assertion.Transitions + "\r\n" + TextBox_Output.Text;
                        }
                        //TextBox_Output.Text += e.Exception.StackTrace + "\r\n" + TextBox_Output.Text;
                        TextBox_Output.Text = e.Exception.Message + "\r\n" + TextBox_Output.Text;
                        TextBox_Output.Text = "Runtime Exception happened during the verification:\r\n" + TextBox_Output.Text;


                        StatusLabel_Text.Text = "Runtime Exception Happened";

                        //MessageBox.Show("Runtime exception!\r\n" +e.Exception.Message,// + "\r\n" + ex.StackTrace,"PAT", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        //this.Close();
                        //return;
                    }
                    else
                    {
                        //if the button is enabled -> the failure is not triggered by cancel action.
                        if (Button_Verify.Enabled)
                        {
                            Common.Ultility.Ultility.LogException(e.Exception, Spec);
                            // SpecProcess.UnLockSharedData();
                            this.Close();
                        }
                        else
                        {
                            this.StatusLabel_Text.Text = "Verification Cancelled";
                        }

                    }
                    Assertion.Clear();
                    //remove the events
                    Assertion.Action -= OnAction;
                    Assertion.ReturnResult -= VerificationFinished;
                    Assertion.Cancelled -= Verification_Cancelled;
                    Assertion.Failed -= MC_Failed;
                    Assertion = null;

                    EnableAllControls();
                }
                catch (Exception ex)
                {
                    Common.Ultility.Ultility.LogException(ex, Spec);
                    //MessageBox.Show("Exception happened: " + ex.Message + "\r\n", // + ex.StackTrace,"PAT", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        //chengbin4 - thread 2?
        private void VerificationFinished()
        {
            if (this.InvokeRequired)
            {
                this.Invoke(new VerificationResultEvents(VerificationFinished));
            }
            else
            {
                try
                {
                    System.Diagnostics.Debug.WriteLine("5555");
                    //timer.Stop();
                    //MCTimer.Stop();

                    long stopMemorySize = System.GC.GetTotalMemory(false);

                    TextBox_Output.Text = "Estimated Memory Used:" + (stopMemorySize - startMemroySize)/1000.0 + "KB\r\n\r\n" + TextBox_Output.Text;
                    //TextBox_Output.Text = "Time Used:" + timer.Elapsed.TotalSeconds + "s\r\n" + TextBox_Output.Text;

                    //TextBox_Output.Text = "\tSearch Depth:" + Assertion.SearchedDepth + "\r\n" + TextBox_Output.Text;

                    if (this.CheckBox_Parallel.Checked && Assertion is AssertionLTL)
                    {
                        AssertionLTL ltl = Assertion as AssertionLTL;
                        TextBox_Output.Text = "SCC Ratio: " + Math.Round(((double) ltl.SCCTotalSize/(double) Assertion.NoOfStates), 2).ToString() + "\r\n" + TextBox_Output.Text; //DataStore.DataManager.GetNoOfStates()
                        if (ltl.SCCCount != 0)
                        {
                            TextBox_Output.Text = "Average SCC Size: " + (ltl.SCCTotalSize/ltl.SCCCount) + "\r\n" + TextBox_Output.Text;
                        }
                        else
                        {
                            TextBox_Output.Text = "Average SCC Size: 0\r\n" + TextBox_Output.Text;
                        }

                        TextBox_Output.Text = "Total SCC states: " + ltl.SCCTotalSize + "\r\n" + TextBox_Output.Text;
                        TextBox_Output.Text = "Number of SCC found: " + ltl.SCCCount + "\r\n" + TextBox_Output.Text;
                    }

                    TextBox_Output.Text = "Total Transitions:" + Assertion.Transitions + "\r\n" + TextBox_Output.Text;
                    string group = "";
                    if (Assertion.NoOfGroupedStates > 0)
                    {
                        group = " (" + Assertion.NoOfGroupedStates + " states are grouped)";
                    }
                    TextBox_Output.Text = "Visited States:" + Assertion.NoOfStates + group + "\r\n" + TextBox_Output.Text; //DataStore.DataManager.GetNoOfStates() 
                    TextBox_Output.Text = "********Verification Statistics********\r\n" + TextBox_Output.Text;



                    //other than reachbility


                    //if (Assertion.VerificationResult == VerificationResultType.UNKNOWN)
                    //{
                    //    ListView_Assertions.SelectedItems[0].ImageIndex = UNKNOWN_ICON;
                    //}
                    ////else if ((Assertion.VerificationResult == VerificationResultType.VALID && Assertion.AssertionType != AssertionType.Reachability) || (Assertion.VerificationResult == VerificationResultType.VALID && Assertion.AssertionType == AssertionType.Reachability && Assertion.BA != null) || (Assertion.VerificationResult == VerificationResultType.INVALID && Assertion.AssertionType == AssertionType.Reachability && Assertion.BA == null))
                    //else if (Assertion.VerificationResult == VerificationResultType.VALID)
                    //{
                    //    ListView_Assertions.SelectedItems[0].ImageIndex = CORRECT_ICON;
                    //}
                    //else if (Assertion.VerificationResult == VerificationResultType.WITHPROBABILITY)
                    //{
                    //    ListView_Assertions.SelectedItems[0].ImageIndex = CORRECT_ICON_PRPB;
                    //}
                    //else
                    //{
                    //    ListView_Assertions.SelectedItems[0].ImageIndex = WRONG_ICON;
                    //}

                    //StatusLabel_Text.Text = "Generating Result...";

                    //Assertion.ReturnResult -= VerificationFinished;
                    //Assertion.ReturnResult += VerificationResult;
                    //Assertion.VerificationMode = false;
                    //Assertion.Start();

                }
                catch (Exception ex)
                {
                    Assertion.Clear();

                    //remove the events
                    Assertion.Action -= OnAction;
                    Assertion.Cancelled -= Verification_Cancelled;
                    Assertion.ReturnResult -= VerificationFinished;
                    Assertion.Failed -= MC_Failed;
                    Spec.AssertionDatabase[Label_SelectedAssertion.Text] = Assertion;

                    Common.Ultility.Ultility.LogException(ex, Spec);
                    //MessageBox.Show("Exception happened during verification: " + ex.Message + "\r\n" + ex.StackTrace, "PAT", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }

        }

        public delegate void VerificationResultEvents();

        //chengbin4 - thread 3?
        public void VerificationResult()//chengbin4 - private
        {
            //if (this.InvokeRequired)
            //{
            //    this.Invoke(new VerificationResultEvents(VerificationResult), null);
            //}
            //else
            {
                try
                {
                    TextBox_Output.Text = Assertion.ResultString + TextBox_Output.Text;

                    StatusLabel_Text.Text = "Verification Completed";

                    TextBox_Output.SelectAll();
                    TextBox_Output.SelectionFont = new Font("Microsoft Sans Serif", 8F, FontStyle.Regular, GraphicsUnit.Point);
                    TextBox_Output.SelectionStart = 0;
                    TextBox_Output.SelectionLength = 0;

                    ColorText(0);

                    TextBox_Output.Select(0, 0);
                    TextBox_Output.ScrollToCaret();

                    //chengbin4 - concern here
                    EnableAllControls();
                    //chengbin4 - problem is that control is passed to UI and UI is reset before exit here

                    MessageBox.Show("Exception happened during verification: ", "PAT", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                catch (Exception ex)
                {
                    Common.Ultility.Ultility.LogException(ex, Spec);
                    //MessageBox.Show("Exception happened during verification: " + ex.Message + "\r\n" + ex.StackTrace, "PAT", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                finally
                {
                    Assertion.Clear();

                    //remove the events
                    Assertion.Action -= OnAction;
                    Assertion.Cancelled -= Verification_Cancelled;
                    Assertion.ReturnResult -= VerificationResult;
                    Assertion.Failed -= MC_Failed;
                    Spec.AssertionDatabase[Label_SelectedAssertion.Text] = Assertion;
                }
            }
        }

        private static Font fnt = new Font("Verdana", 8F, FontStyle.Regular, GraphicsUnit.Point);

        private void ColorText(int start)
        {                       
          
            
            int index = TextBox_Output.Find(mystring, start, -1, RichTextBoxFinds.WholeWord);

            if (index > 0)
            {
                TextBox_Output.SelectionStart = index;
                TextBox_Output.SelectionLength = mystring.Length;
                TextBox_Output.SelectionFont = fnt;
                TextBox_Output.SelectionColor = System.Drawing.Color.Red;
                TextBox_Output.SelectionBackColor = System.Drawing.Color.Yellow;
                ColorText(index + mystring.Length);
            } 
            else
            {
                index = TextBox_Output.Find("-> (", start, -1, RichTextBoxFinds.None);

                if(index > 0)
                {
                    int endIndex = TextBox_Output.Find(")*", start, -1, RichTextBoxFinds.None);
                    if (endIndex > 0)
                    {
                        index = index + 3;
                        if (endIndex - index + 2 > 0)
                        {
                            TextBox_Output.SelectionStart = index;

                            TextBox_Output.SelectionLength = endIndex - index + 2;
                            TextBox_Output.SelectionFont = fnt;
                            TextBox_Output.SelectionColor = System.Drawing.Color.Red;
                            TextBox_Output.SelectionBackColor = System.Drawing.Color.Yellow;
                        }
                        ColorText(endIndex + 2);
                    }
                }                
            }            
        }

        //void TextBox_DataPane_TextChanged(object sender, EventArgs e)
        //{
        //    TextBox_DataPane.Font = new Font(TextBox_DataPane.Font.FontFamily, 8, FontStyle.Regular);
        //    TextBox_DataPane.SelectAll();
        //    TextBox_DataPane.SelectionFont = TextBox_DataPane.Font;
        //    TextBox_DataPane.SelectionStart = 0;
        //    TextBox_DataPane.SelectionLength = 0;
        //}

        public void Verification_Cancelled(object sender, EventArgs e)//chengbin4 - private
        {
            try
            {
                //cancel the verification 
                if(Assertion.VerificationMode)
                {
                    Assertion.Clear();

                    //remove the events
                    Assertion.Action -= OnAction;
                    Assertion.ReturnResult -= VerificationFinished;
                    Assertion.Cancelled -= Verification_Cancelled;
                    Assertion.Failed -= MC_Failed;
                    Assertion = null;

                    StatusLabel_Text.Text = "Verification Cancelled";

                    //set the verification result to be unknow.
                    //ListView_Assertions.SelectedItems[0].ImageIndex = UNKNOWN_ICON;    
                }
                //cancel the result generation
                else
                {
                    TextBox_Output.Text = Assertion.ResultString + TextBox_Output.Text;

                    StatusLabel_Text.Text = "Result Generation Cancelled";
                    ColorText(0);
                    TextBox_Output.Select(0, 0);
                    TextBox_Output.ScrollToCaret();
                    
                }

                EnableAllControls();
                Cursor = Cursors.Default;

            }
            catch (Exception ex)
            {
                Common.Ultility.Ultility.LogException(ex, Spec);
                //MessageBox.Show("Exception happened: " + ex.Message + "\r\n" + ex.StackTrace,"PAT", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        
        private void OnAction(string action)
        {
            TextBox_Output.Text = action + TextBox_Output.Text;            
        }

        public void Button_SimulateCounterExample_Click(object sender, EventArgs e)//chengbin4 - private
        {
            //chengbin4
            //Contract.Requires(Button_SimulateWitnessTrace.Enabled);
            //Contract.Requires(Assertion.CounterExampleTrace != null && Assertion.CounterExampleTrace.Count > 0);
            //Contract.Requires(ListView_Assertions.SelectedItems.Count == 1);

            try
            {
                PAT.Common.GUI.SimulationForm form = new PAT.Common.GUI.SimulationForm();
                form.SetSpec(this.Button_SimulateWitnessTrace.Tag.ToString(), this.Button_SimulateWitnessTrace.Tag.ToString(), this.Spec, Assertion);
                //form.Show();
            }
            catch (Exception ex)
            {
                Common.Ultility.Ultility.LogException(ex, Spec);
                //MessageBox.Show("Exception happened: " + ex.Message + "\r\n" + ex.StackTrace,"PAT", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        public void ListView_Assertions_ItemSelectionChanged(object sender, ListViewItemSelectionChangedEventArgs e)//chengbin4 - private
        {
            try
            {
                //only check the selected item
                if (e.IsSelected)
                {
                    ResetUIOptions(e.Item, true);
                    this.StatusLabel_Text.Text = "Ready";

                }
                else
                {
                    if (ListView_Assertions.SelectedItems.Count == 0)
                    {
                        //set text
                        Label_SelectedAssertion.Text = "";
                        Assertion = null;
                        Button_Verify.Enabled = false;
                        Button_BAGraph.Enabled = false;
                        Button_SimulateWitnessTrace.Enabled = false;
                        this.StatusLabel_Text.Text = "Select an Assertion to Start with";

                    }
                }
            }
            catch (RuntimeException ex)
            {
                Common.Ultility.Ultility.LogRuntimeException(ex);
                this.Close();
            }
            catch (Exception ex)
            {
                Common.Ultility.Ultility.LogException(ex, Spec);
                this.Close();
            }
        }

        //chengbin4 - main control here; reset UI before VerificationResult() exit
        public void ResetUIOptions(ListViewItem item, bool resetUIvalues)//chengbin4 - private
        {
            //chengbin4
            cb_item = item;
            cb_resetUIvalues = resetUIvalues;

            //set text
            Label_SelectedAssertion.Text = item.SubItems[2].Text;
            Assertion = Spec.AssertionDatabase[Label_SelectedAssertion.Text];

            //the select property is an LTL property
            if ((AssertionType)item.Tag == AssertionType.LTL)
            {
                SetFairnessChoices(Assertion.IsProcessLevelFairnessApplicable, Assertion.MustAbstract, resetUIvalues);                
            }
            else
            {
                ComboBox_Fairness.SelectedIndex = 0;
                ComboBox_Fairness.Enabled = false;
            }

            if (Assertion.BA == null)
            {
                Button_BAGraph.Enabled = false;
            }
            else
            {
                Button_BAGraph.Enabled = true;
            }
            

            //if it is deadlock and reachability
            if ((AssertionType)item.Tag == AssertionType.LTL) //|| (AssertionType)item.Tag == AssertionType.Reachability || (AssertionType)item.Tag == AssertionType.LTLSafety
            {
                CheckBox_ShortestTrace.Checked = false;
                CheckBox_ShortestTrace.Enabled = false;                
            }
            else
            {
                CheckBox_ShortestTrace.Enabled = true;
            }

            //verification result handling.                
            //if ((Assertion.VerificationResult == VerificationResultType.INVALID && Assertion.AssertionType != AssertionType.Reachability) || (Assertion.AssertionType == AssertionType.Reachability && Assertion.VerificationResult == VerificationResultType.VALID))
            if (Assertion.CounterExampleTrace != null && Assertion.CounterExampleTrace.Count > 0)
            {
                Button_SimulateWitnessTrace.Enabled = true;
                Button_SimulateWitnessTrace.Tag = Assertion.StartingProcess.ToString().Replace("()", ""); 
            }
            else
            {
                Button_SimulateWitnessTrace.Enabled = false;
            }

                
            Button_Verify.Enabled = true;
        }

        //chengbin4 - cannot change to public because in ResetUIOptions, this is called before setting Button_BAGraph.Enabled
        private void SetFairnessChoices(bool ProcessLevelFairnessApplicable, bool mustAbstract, bool resetUIvalues)
        {
            //if there is no need to refill the fairness option values. 
            if (!resetUIvalues)
            {
                //if the ComboBox_Fairness needs not be to disabled.
                //we need to enable it back.
                //if(!mustAbstract || ProcessLevelFairnessApplicable)
                //{
                ComboBox_Fairness.Enabled = true;
                return;
                //}
            }

            //if (mustAbstract)
            //{
            //    if (ProcessLevelFairnessApplicable)
            //    {
            //        this.ComboBox_Fairness.Items.Clear();
            //        this.ComboBox_Fairness.Items.Add("No Fairness");
            //        this.ComboBox_Fairness.Items.Add("Event Level Weak Fairness");
            //        this.ComboBox_Fairness.Items.Add("Event Level Strong Fairness");
            //        this.ComboBox_Fairness.Items.Add("Process Level Weak Fairness");
            //        this.ComboBox_Fairness.Items.Add("Process Level Strong Fairness");
            //        this.ComboBox_Fairness.Items.Add("Strong Global Fairness");
            //    }
            //    else
            //    {
            //        this.ComboBox_Fairness.Items.Clear();
            //        this.ComboBox_Fairness.Items.Add("No Fairness");
            //        this.ComboBox_Fairness.Items.Add("Event Level Weak Fairness");
            //        this.ComboBox_Fairness.Items.Add("Event Level Strong Fairness");
            //        this.ComboBox_Fairness.Items.Add("Strong Global Fairness");
            //        return;
            //    }                
            //}
            //else
            //{
               
            if (ProcessLevelFairnessApplicable)
            {
                this.ComboBox_Fairness.Items.Clear();
                this.ComboBox_Fairness.Items.Add("No Fairness");
                if (Spec.HasFairEvent)
                {
                    this.ComboBox_Fairness.Items.Add("Fairness Labels Only");
                }
                this.ComboBox_Fairness.Items.Add("Event Level Weak Fairness");
                this.ComboBox_Fairness.Items.Add("Event Level Strong Fairness");
                this.ComboBox_Fairness.Items.Add("Process Level Weak Fairness");
                this.ComboBox_Fairness.Items.Add("Process Level Strong Fairness");
                this.ComboBox_Fairness.Items.Add("Strong Global Fairness");
            }
            else
            {
                this.ComboBox_Fairness.Items.Clear();
                this.ComboBox_Fairness.Items.Add("No Fairness");
                if (Spec.HasFairEvent)
                {
                    this.ComboBox_Fairness.Items.Add("Fairness Labels Only");
                }
                this.ComboBox_Fairness.Items.Add("Event Level Weak Fairness");
                this.ComboBox_Fairness.Items.Add("Event Level Strong Fairness");
                this.ComboBox_Fairness.Items.Add("Strong Global Fairness");
            }
            //}
            ComboBox_Fairness.SelectedIndex = 0;
            ComboBox_Fairness.Enabled = true;

            
        }


        //private void NUD_Depth_ValueChanged(object sender, EventArgs e)
        //{
        //    if (NUD_Depth.Value > 0 || PORDisabled)
        //    {
        //        CheckBox_POR.Checked = false;
        //        CheckBox_POR.Enabled = false;    
        //    }
        //    else
        //    {
        //        CheckBox_POR.Checked = true;
        //        CheckBox_POR.Enabled = true;    
        //    }
        //}

        //private void ComboBox_Fairness_SelectedIndexChanged(object sender, EventArgs e)
        //{
        //    if (this.ComboBox_Fairness.Text.Contains("Strong") || PORDisabled)
        //    {
        //        CheckBox_POR.Checked = false;
        //        CheckBox_POR.Enabled = false;
        //    }
        //    else
        //    {
        //        CheckBox_POR.Checked = true;
        //        CheckBox_POR.Enabled = true;
        //    }                    
        //}



        private void CheckBox_Verbose_CheckedChanged(object sender, EventArgs e)
        {
            try
            {
                if (CheckBox_Verbose.Checked)
                {
                    this.StatusLabel_Text.Text = "Verbose Mode Selected";
                }
                else
                {
                    this.StatusLabel_Text.Text = "Normal Mode Selected";
                }
            }
            catch (Exception ex)
            {
                Common.Ultility.Ultility.LogException(ex, Spec);
            }
        }

        private void CheckBox_POR_CheckedChanged(object sender, EventArgs e)
        {
            //if (CheckBox_POR.Checked)
            //{
            //    this.StatusLabel_Text.Text = "Partial Order Reduction Selected";
            //}
            //else
            //{
            //    this.StatusLabel_Text.Text = "Partial Order Reduction UnSelected";
            //}
        }

        private void CheckBox_Parallel_CheckedChanged(object sender, EventArgs e)
        {
            if (CheckBox_Parallel.Checked)
            {
                this.StatusLabel_Text.Text = "Parallel Verification Selected";
            }
            else
            {
                this.StatusLabel_Text.Text = "Parallel Verification UnSelected";
            }
        }

        private void ModelCheckingForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            //may the exception dialog want to force the application to close, in this case, we should not give up.
            if (Button_Verify.Text == STOP && !ExceptionDialog.ForceExit)
            {
                
                if (Assertion != null)
                {
                    Assertion.Cancel();
                }
                e.Cancel = true;
                //MessageBox.Show("Please wait for the verification to stop first! Otherwise it may cause exceptional behaviors.", Common.Ultility.Ultility.APPLICATION_NAME, MessageBoxButtons.OK);
            }
            
        }

        //private void ModelCheckingForm_FormClosed(object sender, FormClosedEventArgs e)
        //{
        //    ModelCheckingFormInstance = null;
        //}

        //chengbin4**********************************************************************************************
        //[ContractInvariantMethod]
        //protected void ObjectInvariant()
        //{
        //    Contract.Invariant(button_BAGraph_Invariant());
        //    Contract.Invariant(button_SimulateWitnessTrace_Invariant());
        //    Contract.Invariant(comboBox_Fairness_Invariant());
        //    //Contract.Invariant(checkBox_ShortestTrace_Invariant());
        //    //Contract.Invariant(ListView_Assertions.Enabled);
        //}

        private bool button_BAGraph_Invariant()
        {
            if (Label_SelectedAssertion != null && Spec != null && !Label_SelectedAssertion.Text.Equals(""))
            {
                if (Spec.AssertionDatabase[Label_SelectedAssertion.Text].BA == null)
                    return !Button_BAGraph.Enabled;
                else
                    return Button_BAGraph.Enabled;
            }

            return true;
        }

        private bool button_SimulateWitnessTrace_Invariant()
        {
            if (Label_SelectedAssertion != null && Spec != null && !Label_SelectedAssertion.Text.Equals(""))
            {
                if (Spec.AssertionDatabase[Label_SelectedAssertion.Text].CounterExampleTrace != null && Spec.AssertionDatabase[Label_SelectedAssertion.Text].CounterExampleTrace.Count > 0)
                    return Button_SimulateWitnessTrace.Enabled;
                else
                    return !Button_SimulateWitnessTrace.Enabled;
            }

            return true;
        }

        private bool comboBox_Fairness_Invariant()
        {
            if (Label_SelectedAssertion != null && Spec != null && !Label_SelectedAssertion.Text.Equals(""))
            {
                if ((AssertionType)cb_item.Tag == AssertionType.LTL)
                {
                    if (!cb_resetUIvalues)
                        return ComboBox_Fairness.Enabled;

                    if (Spec.AssertionDatabase[Label_SelectedAssertion.Text].IsProcessLevelFairnessApplicable)
                        return ComboBox_Fairness.Items.Contains("No Fairness")
                               && ComboBox_Fairness.Items.Contains("Event Level Weak Fairness")
                               && ComboBox_Fairness.Items.Contains("Event Level Strong Fairness")
                               && ComboBox_Fairness.Items.Contains("Process Level Weak Fairness")
                               && ComboBox_Fairness.Items.Contains("Process Level Strong Fairness")
                               && ComboBox_Fairness.Items.Contains("Strong Global Fairness")
                               && ComboBox_Fairness.Enabled && ComboBox_Fairness.SelectedIndex == 0
                               &&
                               ((!Spec.HasFairEvent && !ComboBox_Fairness.Items.Contains("Fairness Labels Only")) ||
                                (Spec.HasFairEvent && ComboBox_Fairness.Items.Contains("Fairness Labels Only")));
                    else
                        return ComboBox_Fairness.Items.Contains("No Fairness")
                               && ComboBox_Fairness.Items.Contains("Event Level Weak Fairness")
                               && ComboBox_Fairness.Items.Contains("Event Level Strong Fairness")
                               && !ComboBox_Fairness.Items.Contains("Process Level Weak Fairness")
                               && !ComboBox_Fairness.Items.Contains("Process Level Strong Fairness")
                               && ComboBox_Fairness.Items.Contains("Strong Global Fairness")
                               && ComboBox_Fairness.Enabled && ComboBox_Fairness.SelectedIndex == 0
                               &&
                               ((!Spec.HasFairEvent && !ComboBox_Fairness.Items.Contains("Fairness Labels Only")) ||
                                (Spec.HasFairEvent && ComboBox_Fairness.Items.Contains("Fairness Labels Only")));

                }
                else
                    return !ComboBox_Fairness.Enabled && ComboBox_Fairness.SelectedIndex == 0;
            }

            return true;
        }

        private bool checkBox_ShortestTrace_Invariant()
        {
            if (Label_SelectedAssertion != null && Spec != null && !Label_SelectedAssertion.Text.Equals(""))
            {
                if ((AssertionType)cb_item.Tag == AssertionType.LTL)
                    return !CheckBox_ShortestTrace.Checked && !CheckBox_ShortestTrace.Enabled;
                else
                    return CheckBox_ShortestTrace.Enabled;
            }

            return true;
        }
        //chengbin4**********************************************************************************************

    }
}